# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/CanvasGroupy',
                'doc_host': 'https://FleischerResearchLab.github.io',
                'git_url': 'https://github.com/FleischerResearchLab/CanvasGroupy',
                'lib_path': 'CanvasGroupy'},
  'syms': { 'CanvasGroupy.assign': { 'CanvasGroupy.assign.AssignGroup': ('api/groupeng_assign.html#assigngroup', 'CanvasGroupy/assign.py'),
                                     'CanvasGroupy.assign.AssignGroup.__init__': ( 'api/groupeng_assign.html#assigngroup.__init__',
                                                                                   'CanvasGroupy/assign.py'),
                                     'CanvasGroupy.assign.AssignGroup.assign_groups': ( 'api/groupeng_assign.html#assigngroup.assign_groups',
                                                                                        'CanvasGroupy/assign.py'),
                                     'CanvasGroupy.assign.AssignGroup.create_canvas_group': ( 'api/groupeng_assign.html#assigngroup.create_canvas_group',
                                                                                              'CanvasGroupy/assign.py'),
                                     'CanvasGroupy.assign.AssignGroup.create_github_group': ( 'api/groupeng_assign.html#assigngroup.create_github_group',
                                                                                              'CanvasGroupy/assign.py')},
            'CanvasGroupy.canvas': { 'CanvasGroupy.canvas.CanvasGroup': ('api/canvas.html#canvasgroup', 'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.__init__': ( 'api/canvas.html#canvasgroup.__init__',
                                                                                   'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup._check_single_github_username': ( 'api/canvas.html#canvasgroup._check_single_github_username',
                                                                                                        'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup._progress': ( 'api/canvas.html#canvasgroup._progress',
                                                                                    'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.assign_canvas_group': ( 'api/canvas.html#canvasgroup.assign_canvas_group',
                                                                                              'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.auth_canvas': ( 'api/canvas.html#canvasgroup.auth_canvas',
                                                                                      'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.check_github_usernames': ( 'api/canvas.html#canvasgroup.check_github_usernames',
                                                                                                 'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.create_conversation': ( 'api/canvas.html#canvasgroup.create_conversation',
                                                                                              'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.create_group': ( 'api/canvas.html#canvasgroup.create_group',
                                                                                       'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.create_group_category': ( 'api/canvas.html#canvasgroup.create_group_category',
                                                                                                'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.fetch_username_from_quiz': ( 'api/canvas.html#canvasgroup.fetch_username_from_quiz',
                                                                                                   'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.get_course': ( 'api/canvas.html#canvasgroup.get_course',
                                                                                     'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.get_email_by_name': ( 'api/canvas.html#canvasgroup.get_email_by_name',
                                                                                            'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.get_group_categories': ( 'api/canvas.html#canvasgroup.get_group_categories',
                                                                                               'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.get_groups': ( 'api/canvas.html#canvasgroup.get_groups',
                                                                                     'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.join_canvas_group': ( 'api/canvas.html#canvasgroup.join_canvas_group',
                                                                                            'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.link_assignment': ( 'api/canvas.html#canvasgroup.link_assignment',
                                                                                          'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.post_grade': ( 'api/canvas.html#canvasgroup.post_grade',
                                                                                     'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.set_course': ( 'api/canvas.html#canvasgroup.set_course',
                                                                                     'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.CanvasGroup.set_group_category': ( 'api/canvas.html#canvasgroup.set_group_category',
                                                                                             'CanvasGroupy/canvas.py'),
                                     'CanvasGroupy.canvas.bcolors': ('api/canvas.html#bcolors', 'CanvasGroupy/canvas.py')},
            'CanvasGroupy.github': { 'CanvasGroupy.github.GitHubGroup': ('api/github.html#githubgroup', 'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.__init__': ( 'api/github.html#githubgroup.__init__',
                                                                                   'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.add_collaborator': ( 'api/github.html#githubgroup.add_collaborator',
                                                                                           'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.add_team': ( 'api/github.html#githubgroup.add_team',
                                                                                   'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.auth_github': ( 'api/github.html#githubgroup.auth_github',
                                                                                      'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.create_feedback_dir': ( 'api/github.html#githubgroup.create_feedback_dir',
                                                                                              'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.create_group_repo': ( 'api/github.html#githubgroup.create_group_repo',
                                                                                            'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.create_issue': ( 'api/github.html#githubgroup.create_issue',
                                                                                       'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.create_issue_from_md': ( 'api/github.html#githubgroup.create_issue_from_md',
                                                                                               'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.create_repo': ( 'api/github.html#githubgroup.create_repo',
                                                                                      'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.get_org_repo': ( 'api/github.html#githubgroup.get_org_repo',
                                                                                       'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.get_repo': ( 'api/github.html#githubgroup.get_repo',
                                                                                   'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.get_team': ( 'api/github.html#githubgroup.get_team',
                                                                                   'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.release_feedback': ( 'api/github.html#githubgroup.release_feedback',
                                                                                           'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.remove_collaborator': ( 'api/github.html#githubgroup.remove_collaborator',
                                                                                              'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.rename_files': ( 'api/github.html#githubgroup.rename_files',
                                                                                       'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.resend_invitations': ( 'api/github.html#githubgroup.resend_invitations',
                                                                                             'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.resent_invitations_team_repos': ( 'api/github.html#githubgroup.resent_invitations_team_repos',
                                                                                                        'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.GitHubGroup.set_org': ( 'api/github.html#githubgroup.set_org',
                                                                                  'CanvasGroupy/github.py'),
                                     'CanvasGroupy.github.bcolors': ('api/github.html#bcolors', 'CanvasGroupy/github.py')},
            'CanvasGroupy.grading': { 'CanvasGroupy.grading.Grading': ('api/project_grading.html#grading', 'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.Grading.__init__': ( 'api/project_grading.html#grading.__init__',
                                                                                 'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.Grading.create_issue_from_md': ( 'api/project_grading.html#grading.create_issue_from_md',
                                                                                             'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.Grading.fetch_issue': ( 'api/project_grading.html#grading.fetch_issue',
                                                                                    'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.Grading.grade_project': ( 'api/project_grading.html#grading.grade_project',
                                                                                      'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.Grading.parse_score_from_issue': ( 'api/project_grading.html#grading.parse_score_from_issue',
                                                                                               'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.Grading.update_canvas_score': ( 'api/project_grading.html#grading.update_canvas_score',
                                                                                            'CanvasGroupy/grading.py'),
                                      'CanvasGroupy.grading.bcolors': ('api/project_grading.html#bcolors', 'CanvasGroupy/grading.py')}}}
